@@ -37,9 +37,8 @@ the classifier is trained with lr = 1e-4.
 Note:
     See: https://pytorch.org/tutorials/beginner/transfer_learning_tutorial.html
 """
-import argparse
+
 import logging
-import os
 from pathlib import Path
 from typing import Union
 
@@ -49,6 +48,7 @@ from torch import nn, optim
 from torch.optim.lr_scheduler import MultiStepLR
 from torch.optim.optimizer import Optimizer
 from torch.utils.data import DataLoader
+from torchmetrics import Accuracy
 from torchvision import models, transforms
 from torchvision.datasets import ImageFolder
 from torchvision.datasets.utils import download_and_extract_archive
@@ -58,6 +58,7 @@ from pl_examples import cli_lightning_logo
 from pytorch_lightning import LightningDataModule
 from pytorch_lightning.callbacks.finetuning import BaseFinetuning
 from pytorch_lightning.utilities import rank_zero_info
+from pytorch_lightning.utilities.cli import LightningCLI
 
 log = logging.getLogger(__name__)
 DATA_URL = "https://storage.googleapis.com/mledu-datasets/cats_and_dogs_filtered.zip"
@@ -68,6 +69,7 @@ DATA_URL = "https://storage.googleapis.com/mledu-datasets/cats_and_dogs_filtered
 class MilestonesFinetuning(BaseFinetuning):
 
     def __init__(self, milestones: tuple = (5, 10), train_bn: bool = False):
+        super().__init__()
         self.milestones = milestones
         self.train_bn = train_bn
 
@@ -92,10 +94,17 @@ class CatDogImageDataModule(LightningDataModule):
 
     def __init__(
         self,
-        dl_path: Union[str, Path],
+        dl_path: Union[str, Path] = "data",
         num_workers: int = 0,
         batch_size: int = 8,
     ):
+        """CatDogImageDataModule
+
+        Args:
+            dl_path: root directory where to download the data
+            num_workers: number of CPU workers
+            batch_size: number of sample in a batch
+        """
         super().__init__()
 
         self._dl_path = dl_path
@@ -145,17 +154,6 @@ class CatDogImageDataModule(LightningDataModule):
         log.info("Validation data loaded.")
         return self.__dataloader(train=False)
 
-    @staticmethod
-    def add_model_specific_args(parent_parser):
-        parser = argparse.ArgumentParser(parents=[parent_parser])
-        parser.add_argument(
-            "--num-workers", default=0, type=int, metavar="W", help="number of CPU workers", dest="num_workers"
-        )
-        parser.add_argument(
-            "--batch-size", default=8, type=int, metavar="W", help="number of sample in a batch", dest="batch_size"
-        )
-        return parser
-
 
 #  --- Pytorch-lightning module ---
 
@@ -165,17 +163,22 @@ class TransferLearningModel(pl.LightningModule):
     def __init__(
         self,
         backbone: str = "resnet50",
-        train_bn: bool = True,
-        milestones: tuple = (5, 10),
+        train_bn: bool = False,
+        milestones: tuple = (2, 4),
         batch_size: int = 32,
-        lr: float = 1e-2,
+        lr: float = 1e-3,
         lr_scheduler_gamma: float = 1e-1,
         num_workers: int = 6,
         **kwargs,
     ) -> None:
-        """
+        """TransferLearningModel
+
         Args:
-            dl_path: Path where the data will be downloaded
+            backbone: Name (as in ``torchvision.models``) of the feature extractor
+            train_bn: Whether the BatchNorm layers should be trainable
+            milestones: List of two epochs milestones
+            lr: Initial learning rate
+            lr_scheduler_gamma: Factor by which the learning rate is reduced at each milestone
         """
         super().__init__()
         self.backbone = backbone
@@ -188,8 +191,8 @@ class TransferLearningModel(pl.LightningModule):
 
         self.__build_model()
 
-        self.train_acc = pl.metrics.Accuracy()
-        self.valid_acc = pl.metrics.Accuracy()
+        self.train_acc = Accuracy()
+        self.valid_acc = Accuracy()
         self.save_hyperparameters()
 
     def __build_model(self):
@@ -224,7 +227,7 @@ class TransferLearningModel(pl.LightningModule):
         # 2. Classifier (returns logits):
         x = self.fc(x)
 
-        return torch.sigmoid(x)
+        return x
 
     def loss(self, logits, labels):
         return self.loss_func(input=logits, target=labels)
@@ -233,13 +236,14 @@ class TransferLearningModel(pl.LightningModule):
         # 1. Forward pass:
         x, y = batch
         y_logits = self.forward(x)
+        y_scores = torch.sigmoid(y_logits)
         y_true = y.view((-1, 1)).type_as(x)
 
         # 2. Compute loss
         train_loss = self.loss(y_logits, y_true)
 
         # 3. Compute accuracy:
-        self.log("train_acc", self.train_acc(y_logits, y_true.int()), prog_bar=True)
+        self.log("train_acc", self.train_acc(y_scores, y_true.int()), prog_bar=True)
 
         return train_loss
 
@@ -247,13 +251,14 @@ class TransferLearningModel(pl.LightningModule):
         # 1. Forward pass:
         x, y = batch
         y_logits = self.forward(x)
+        y_scores = torch.sigmoid(y_logits)
         y_true = y.view((-1, 1)).type_as(x)
 
         # 2. Compute loss
         self.log("val_loss", self.loss(y_logits, y_true), prog_bar=True)
 
         # 3. Compute accuracy:
-        self.log("val_acc", self.valid_acc(y_logits, y_true.int()), prog_bar=True)
+        self.log("val_acc", self.valid_acc(y_scores, y_true.int()), prog_bar=True)
 
     def configure_optimizers(self):
         parameters = list(self.parameters())
@@ -266,90 +271,31 @@ class TransferLearningModel(pl.LightningModule):
         scheduler = MultiStepLR(optimizer, milestones=self.milestones, gamma=self.lr_scheduler_gamma)
         return [optimizer], [scheduler]
 
-    @staticmethod
-    def add_model_specific_args(parent_parser):
-        parser = argparse.ArgumentParser(parents=[parent_parser])
-        parser.add_argument(
-            "--backbone",
-            default="resnet50",
-            type=str,
-            metavar="BK",
-            help="Name (as in ``torchvision.models``) of the feature extractor",
-        )
-        parser.add_argument(
-            "--epochs", default=15, type=int, metavar="N", help="total number of epochs", dest="nb_epochs"
-        )
-        parser.add_argument("--batch-size", default=8, type=int, metavar="B", help="batch size", dest="batch_size")
-        parser.add_argument("--gpus", type=int, default=0, help="number of gpus to use")
-        parser.add_argument(
-            "--lr", "--learning-rate", default=1e-3, type=float, metavar="LR", help="initial learning rate", dest="lr"
-        )
-        parser.add_argument(
-            "--lr-scheduler-gamma",
-            default=1e-1,
-            type=float,
-            metavar="LRG",
-            help="Factor by which the learning rate is reduced at each milestone",
-            dest="lr_scheduler_gamma",
-        )
-        parser.add_argument(
-            "--train-bn",
-            default=False,
-            type=bool,
-            metavar="TB",
-            help="Whether the BatchNorm layers should be trainable",
-            dest="train_bn",
-        )
-        parser.add_argument(
-            "--milestones", default=[2, 4], type=list, metavar="M", help="List of two epochs milestones"
-        )
-        return parser
+
+class MyLightningCLI(LightningCLI):
+
+    def add_arguments_to_parser(self, parser):
+        parser.add_class_arguments(MilestonesFinetuning, 'finetuning')
+        parser.link_arguments('data.batch_size', 'model.batch_size')
+        parser.link_arguments('finetuning.milestones', 'model.milestones')
+        parser.link_arguments('finetuning.train_bn', 'model.train_bn')
+        parser.set_defaults({
+            'trainer.max_epochs': 15,
+            'trainer.weights_summary': None,
+            'trainer.progress_bar_refresh_rate': 1,
+            'trainer.num_sanity_val_steps': 0,
+        })
+
+    def instantiate_trainer(self):
+        finetuning_callback = MilestonesFinetuning(**self.config_init['finetuning'])
+        self.trainer_defaults['callbacks'] = [finetuning_callback]
+        super().instantiate_trainer()
 
 
-def main(args: argparse.Namespace) -> None:
-    """Train the model.
-
-    Args:
-        args: Model hyper-parameters
-
-    Note:
-        For the sake of the example, the images dataset will be downloaded
-        to a temporary directory.
-    """
-
-    datamodule = CatDogImageDataModule(
-        dl_path=os.path.join(args.root_data_path, 'data'), batch_size=args.batch_size, num_workers=args.num_workers
-    )
-    model = TransferLearningModel(**vars(args))
-    finetuning_callback = MilestonesFinetuning(milestones=args.milestones)
-
-    trainer = pl.Trainer(
-        weights_summary=None,
-        progress_bar_refresh_rate=1,
-        num_sanity_val_steps=0,
-        gpus=args.gpus,
-        max_epochs=args.nb_epochs,
-        callbacks=[finetuning_callback]
-    )
-
-    trainer.fit(model, datamodule=datamodule)
-
-
-def get_args() -> argparse.Namespace:
-    parent_parser = argparse.ArgumentParser(add_help=False)
-    parent_parser.add_argument(
-        "--root-data-path",
-        metavar="DIR",
-        type=str,
-        default=Path.cwd().as_posix(),
-        help="Root directory where to download the data",
-        dest="root_data_path",
-    )
-    parser = TransferLearningModel.add_model_specific_args(parent_parser)
-    parser = CatDogImageDataModule.add_argparse_args(parser)
-    return parser.parse_args()
+def cli_main():
+    MyLightningCLI(TransferLearningModel, CatDogImageDataModule, seed_everything_default=1234)
 
 
 if __name__ == "__main__":
     cli_lightning_logo()
-    main(get_args())
+    cli_main()
