@@ -12,43 +12,51 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import numbers
-from typing import Any
+from typing import Dict, Optional
 
 import torch
 from torchmetrics import Metric
 
+from pytorch_lightning.utilities.exceptions import MisconfigurationException
+from pytorch_lightning.utilities.types import _METRIC
+
 
 class MetricsHolder:
     """
-    This class acts as a dictonary holder.
+    This class acts as a dictionary holder.
     It holds metrics and implements conversion functions.
     Those functions will be triggered within LoggerConnector
     when the property is being requested from the user.
     """
 
-    def __init__(self, to_float: bool = False):
-        self.metrics = {}
+    def __init__(self, to_float: bool = False) -> None:
+        self.metrics: Dict[str, _METRIC] = {}
         self._to_float = to_float
 
-    def update(self, metrics):
+    def update(self, metrics: dict) -> None:
         self.metrics.update(metrics)
 
-    def pop(self, key, default):
+    def pop(self, key: str, default: _METRIC) -> _METRIC:
         return self.metrics.pop(key, default)
 
-    def reset(self, metrics):
+    def reset(self, metrics: Dict[str, _METRIC]) -> None:
         self.metrics = metrics
 
-    def convert(self, use_tpu: bool, device: torch.device):
+    def convert(self, device: Optional[torch.device]) -> None:
         for key, value in self.metrics.items():
-            self.metrics[key] = self._convert(value, use_tpu, device)
+            if self._to_float:
+                if isinstance(value, torch.Tensor) and value.numel() != 1:
+                    raise MisconfigurationException(
+                        f"The metric `{key}` does not contain a single element"
+                        f" thus it cannot be converted to float. Found `{value}`"
+                    )
+                converted = self._convert_to_float(value)
+            else:
+                converted = self._convert_to_tensor(value, device)
+            self.metrics[key] = converted
 
-    def _convert(self, current: Any, use_tpu: bool, device: torch.device):
-        if self._to_float:
-            return self._convert_to_float(current, use_tpu, device)
-        return self._convert_to_tensor(current, use_tpu, device)
-
-    def _convert_to_float(self, current, use_tpu: bool, device: torch.device):
+    @staticmethod
+    def _convert_to_float(current: _METRIC) -> float:
         if isinstance(current, Metric):
             current = current.compute().detach()
 
@@ -60,16 +68,13 @@ class MetricsHolder:
 
         return current
 
-    def _convert_to_tensor(self, current: Any, use_tpu: bool, device: torch.device):
-        if current is not None:
-            if isinstance(current, Metric):
-                current = current.compute().detach()
+    @staticmethod
+    def _convert_to_tensor(current: _METRIC, device: Optional[torch.device]) -> torch.Tensor:
+        if isinstance(current, Metric):
+            current = current.compute().detach()
 
-            elif isinstance(current, numbers.Number):
-                if device is None:
-                    current = torch.tensor(current, dtype=torch.float)
-                else:
-                    current = torch.tensor(current, device=device, dtype=torch.float)
+        elif isinstance(current, numbers.Number):
+            current = torch.tensor(current, device=device, dtype=torch.float)
 
         if isinstance(current, torch.Tensor) and current.device.type == "xla":
             current = current.cpu()
